@@ -1,9 +1,9 @@
 /* apps/req.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -60,10 +60,12 @@
 #include <stdlib.h>
 #include <time.h>
 #include <string.h>
+#ifdef WIN16
+#define APPS_WIN16
+#endif
 #include "apps.h"
-#include "buffer.h"
-#include "rsa.h"
-#include "envelope.h"
+#include "bio.h"
+#include "evp.h"
 #include "rand.h"
 #include "conf.h"
 #include "err.h"
@@ -97,15 +99,15 @@
  * -key file	- make a request using key in file (or use it for verification).
  * -keyform	- key file format.
  * -newkey	- make a key and a request.
- * -modulus	- print the RSA modulus
+ * -modulus	- print RSA modulus.
  * -x509	- output a self signed X509 structure instead.
  * -asn1-kludge	- output new certificate request in a format that some CA's
  *		  require.  This format is wrong
  */
 
 #ifndef NOPROTO
-static int make_REQ(X509_REQ *req,RSA *rsa,int attribs);
-static int add_attribute_object(X509_NAME *n, char *text, char *def, 
+static int make_REQ(X509_REQ *req,EVP_PKEY *pkey,int attribs);
+static int add_attribute_object(STACK *n, char *text, char *def, 
 	char *value, int nid,int min,int max);
 static int add_DN_object(X509_NAME *n, char *text, char *def, char *value,
 	int nid,int min,int max);
@@ -125,26 +127,35 @@ static LHASH *config=NULL;
 #endif
 static LHASH *req_conf=NULL;
 
+#define TYPE_RSA	1
+#define TYPE_DSA	2
+#define TYPE_DH		3
+
 int MAIN(argc, argv)
 int argc;
 char **argv;
 	{
+	DSA *dsa_params=NULL;
 	int ex=1,x509=0,days=30;
 	X509 *x509ss=NULL;
 	X509_REQ *req=NULL;
-	RSA *rsa=NULL;
-	int i,badops=0,newreq=0,newkey= -1;
+	EVP_PKEY *pkey=NULL;
+	int i,badops=0,newreq=0,newkey= -1,pkey_type=0;
 	BIO *in=NULL,*out=NULL;
 	int informat,outformat,verify=0,noout=0,text=0,keyform=FORMAT_PEM;
-	int nodes=0,kludge=1;
+	int nodes=0,kludge=0;
 	char *infile,*outfile,*prog,*keyfile=NULL,*template=NULL,*keyout=NULL;
-	EVP_CIPHER *cipher=EVP_des_ede3_cbc();
+	EVP_CIPHER *cipher=NULL;
 	int modulus=0;
 	char *p;
+	EVP_MD *md_alg=NULL,*digest=EVP_md5();
 #ifndef MONOLITH
 	MS_STATIC char config_name[256];
 #endif
 
+#ifndef NO_DES
+	cipher=EVP_des_ede3_cbc();
+#endif
 	apps_startup();
 
 	if (bio_err == NULL)
@@ -177,7 +188,10 @@ char **argv;
 			keyfile= *(++argv);
 			}
 		else if (strcmp(*argv,"-new") == 0)
+			{
+			pkey_type=TYPE_RSA;
 			newreq=1;
+			}
 		else if (strcmp(*argv,"-config") == 0)
 			{	
 			if (--argc < 1) goto bad;
@@ -206,7 +220,58 @@ char **argv;
 		else if (strcmp(*argv,"-newkey") == 0)
 			{
 			if (--argc < 1) goto bad;
-			newkey= atoi(*(++argv));
+			p= *(++argv);
+			if ((strncmp("rsa:",p,4) == 0) ||
+				((p[0] >= '0') && (p[0] <= '9')))
+				{
+				pkey_type=TYPE_RSA;
+				p+=4;
+				newkey= atoi(p);
+				}
+			else if (strncmp("dsa:",p,4) == 0)
+				{
+				X509 *xtmp=NULL;
+				EVP_PKEY *dtmp;
+
+				pkey_type=TYPE_DSA;
+				p+=4;
+				if ((in=BIO_new_file(p,"r")) == NULL)
+					{
+					perror(p);
+					goto end;
+					}
+				if ((dsa_params=PEM_read_bio_DSAparams(in,NULL,NULL)) == NULL)
+					{
+					ERR_clear_error();
+					BIO_reset(in);
+					if ((xtmp=PEM_read_bio_X509(in,NULL,NULL)) == NULL)
+						{
+						BIO_printf(bio_err,"unable to load DSA parameters from file\n");
+						goto end;
+						}
+					dtmp=X509_get_pubkey(xtmp);
+					if (dtmp->type == EVP_PKEY_DSA)
+						dsa_params=DSAparams_dup(dtmp->pkey.dsa);
+					X509_free(xtmp);
+					if (dsa_params == NULL)
+						{
+						BIO_printf(bio_err,"Certificate does not contain DSA parameters\n");
+						goto end;
+						}
+					
+					}
+				BIO_free(in);
+				newkey=BN_num_bits(dsa_params->p);
+				in=NULL;
+				}
+			else if (strncmp("dh:",p,4) == 0)
+				{
+				pkey_type=TYPE_DH;
+				p+=3;
+				}
+			else
+				pkey_type=TYPE_RSA;
+
 			newreq=1;
 			}
 		else if (strcmp(*argv,"-modulus") == 0)
@@ -223,13 +288,21 @@ char **argv;
 			x509=1;
 		else if (strcmp(*argv,"-asn1-kludge") == 0)
 			kludge=1;
+		else if (strcmp(*argv,"-no-asn1-kludge") == 0)
+			kludge=0;
 		else if (strcmp(*argv,"-days") == 0)
 			{
 			if (--argc < 1) goto bad;
 			days= atoi(*(++argv));
 			if (days == 0) days=30;
 			}
+		else if ((md_alg=EVP_get_digestbyname(&((*argv)[1]))) != NULL)
+			{
+			/* ok */
+			digest=md_alg;
+			}
 		else
+
 			{
 			BIO_printf(bio_err,"unknown option %s\n",*argv);
 			badops=1;
@@ -248,22 +321,25 @@ bad:
 		BIO_printf(bio_err," -outform arg   output format - one of DER TXT PEM\n");
 		BIO_printf(bio_err," -in arg        inout file\n");
 		BIO_printf(bio_err," -out arg       output file\n");
-		BIO_printf(bio_err," -modulus       print the RSA public key modulus\n");
 		BIO_printf(bio_err," -text          text form of request\n");
 		BIO_printf(bio_err," -noout         do not output REQ\n");
 		BIO_printf(bio_err," -verify        verify signature on REQ\n");
+		BIO_printf(bio_err," -modulus       RSA modulus\n");
 		BIO_printf(bio_err," -nodes         don't encrypt the output key\n");
-		BIO_printf(bio_err," -key file	using the RSA key contained in file\n");
-		BIO_printf(bio_err," -keyform arg	key file format\n");
-		BIO_printf(bio_err," -newkey bits	generate a new key\n");
+		BIO_printf(bio_err," -key file	use the private key contained in file\n");
+		BIO_printf(bio_err," -keyform arg   key file format\n");
 		BIO_printf(bio_err," -keyout arg    file to send the key to\n");
+		BIO_printf(bio_err," -newkey rsa:bits generate a new RSA key of 'bits' in size\n");
+		BIO_printf(bio_err," -newkey dsa:file generate a new DSA key, parameters taken from CA in 'file'\n");
+
+		BIO_printf(bio_err," -[digest]      Digest to sign with (md5, sha1, md2, mdc2)\n");
 		BIO_printf(bio_err," -config file   request templace file.\n");
 		BIO_printf(bio_err," -new           new request.\n");
 		BIO_printf(bio_err," -x509          output a x509 structure instead of a cert. req.\n");
 		BIO_printf(bio_err," -days          number of days a x509 generated by -x509 is valid for.\n");
 		BIO_printf(bio_err," -asn1-kludge   Output the 'request' in a format that is wrong but some CA's\n");
 		BIO_printf(bio_err,"                have been reported as requiring\n");
-		BIO_printf(bio_err,"                [ It is now always turned on ]\n");
+		BIO_printf(bio_err,"                [ It is now always turned on but can be turned off with -no-asn1-kludge ]\n");
 		goto end;
 		}
 
@@ -292,7 +368,6 @@ bad:
 		if (req_conf == NULL)
 			{
 			BIO_printf(bio_err,"error on line %ld of %s\n",errline,template);
-			ERR_print_errors(bio_err);
 			goto end;
 			}
 		}
@@ -307,13 +382,17 @@ bad:
 			}
 		}
 
+	if ((md_alg == NULL) &&
+		((p=CONF_get_string(req_conf,SECTION,"default_md")) != NULL))
+		{
+		if ((md_alg=EVP_get_digestbyname(p)) != NULL)
+			digest=md_alg;
+		}
+
 	in=BIO_new(BIO_s_file());
 	out=BIO_new(BIO_s_file());
 	if ((in == NULL) || (out == NULL))
-		{
-		ERR_print_errors(bio_err);
 		goto end;
-		}
 
 	if (keyfile != NULL)
 		{
@@ -323,25 +402,25 @@ bad:
 			goto end;
 			}
 
-		if (keyform == FORMAT_ASN1)
+/*		if (keyform == FORMAT_ASN1)
 			rsa=d2i_RSAPrivateKey_bio(in,NULL);
-		else if (keyform == FORMAT_PEM)
-			rsa=PEM_read_bio_RSAPrivateKey(in,NULL,NULL);
+		else */
+		if (keyform == FORMAT_PEM)
+			pkey=PEM_read_bio_PrivateKey(in,NULL,NULL);
 		else
 			{
 			BIO_printf(bio_err,"bad input format specified for X509 request\n");
 			goto end;
 			}
 
-		if (rsa == NULL)
+		if (pkey == NULL)
 			{
-			BIO_printf(bio_err,"unable to load RSA key\n");
-			ERR_print_errors(bio_err);
+			BIO_printf(bio_err,"unable to load Private key\n");
 			goto end;
 			}
 		}
 
-	if (newreq && (rsa == NULL))
+	if (newreq && (pkey == NULL))
 		{
 		char *randfile;
 		char buffer[200];
@@ -368,29 +447,40 @@ bad:
 			if (newkey <= 0)
 				newkey=DEFAULT_KEY_LENGTH;
 			}
+
 		if (newkey < MIN_KEY_LENGTH)
 			{
-			BIO_printf(bio_err,"RSA private key length is too short,\n");
+			BIO_printf(bio_err,"private key length is too short,\n");
 			BIO_printf(bio_err,"it needs to be at least %d bits, not %d\n",MIN_KEY_LENGTH,newkey);
 			goto end;
 			}
-		BIO_printf(bio_err,"Generating a %d bit private key\n",newkey);
-		rsa=RSA_generate_key(newkey,0x10001,req_cb);
+		BIO_printf(bio_err,"Generating a %d bit %s private key\n",
+			newkey,(pkey_type == TYPE_RSA)?"RSA":"DSA");
+
+		if ((pkey=EVP_PKEY_new()) == NULL) goto end;
+
+#ifndef NO_RSA
+		if (pkey_type == TYPE_RSA)
+			{
+			if (!EVP_PKEY_assign_RSA(pkey,
+				RSA_generate_key(newkey,0x10001,req_cb)))
+				goto end;
+			}
+		else
+#endif
+#ifndef NO_DSA
+			if (pkey_type == TYPE_DSA)
+			{
+			if (!DSA_generate_key(dsa_params)) goto end;
+			if (!EVP_PKEY_assign_DSA(pkey,dsa_params)) goto end;
+			dsa_params=NULL;
+			}
+#endif
+
 		if ((randfile == NULL) || (RAND_write_file(randfile) == 0))
 			BIO_printf(bio_err,"unable to write 'random state'\n");
 
-		if (rsa == NULL)
-			{
-			ERR_print_errors(bio_err);
-			goto end;
-			}
-
-		if (RSA_size(rsa)*8 < MIN_KEY_LENGTH)
-			{
-			BIO_printf(bio_err,"RSA private key length is too short,\n");
-			BIO_printf(bio_err,"it needs to be at least %d bits, not %d\n",MIN_KEY_LENGTH,newkey);
-			goto end;
-			}
+		if (pkey == NULL) goto end;
 
 		if (keyout == NULL)
 			keyout=CONF_get_string(req_conf,SECTION,KEYFILE);
@@ -411,23 +501,24 @@ bad:
 			}
 
 		p=CONF_get_string(req_conf,SECTION,"encrypt_rsa_key");
+		if (p == NULL)
+			p=CONF_get_string(req_conf,SECTION,"encrypt_key");
 		if ((p != NULL) && (strcmp(p,"no") == 0))
 			cipher=NULL;
 		if (nodes) cipher=NULL;
 		
 		i=0;
 loop:
-		if (!PEM_write_bio_RSAPrivateKey(out,rsa,cipher,
+		if (!PEM_write_bio_PrivateKey(out,pkey,cipher,
 			NULL,0,NULL))
 			{
 			if ((ERR_GET_REASON(ERR_peek_error()) ==
-				PEM_R_READ_KEY) && (i < 3))
+				PEM_R_PROBLEMS_GETTING_PASSWORD) && (i < 3))
 				{
 				ERR_clear_error();
 				i++;
 				goto loop;
 				}
-			ERR_print_errors(bio_err);
 			goto end;
 			}
 		BIO_printf(bio_err,"-----\n");
@@ -462,14 +553,16 @@ loop:
 		if (req == NULL)
 			{
 			BIO_printf(bio_err,"unable to load X509 request\n");
-			ERR_print_errors(bio_err);
 			goto end;
 			}
 		}
 
 	if (newreq || x509)
 		{
-		if (rsa == NULL)
+		if (pkey->type == EVP_PKEY_DSA)
+			digest=EVP_dss1();
+
+		if (pkey == NULL)
 			{
 			BIO_printf(bio_err,"you need to specify a private key\n");
 			goto end;
@@ -479,100 +572,67 @@ loop:
 			req=X509_REQ_new();
 			if (req == NULL)
 				{
-				ERR_print_errors(bio_err);
 				goto end;
 				}
 
-			i=make_REQ(req,rsa,!x509);
+			i=make_REQ(req,pkey,!x509);
 			if (kludge >= 0)
 				req->req_info->req_kludge=kludge;
 			if (!i)
 				{
 				BIO_printf(bio_err,"problems making Certificate Request\n");
-				ERR_print_errors(bio_err);
 				goto end;
 				}
 			}
 		if (x509)
 			{
-			EVP_PKEY pkey;
-			X509_CINF *ci;
-			X509_REQ_INFO *ri;
-
 			if ((x509ss=X509_new()) == NULL) goto end;
-			ci=x509ss->cert_info;
-			ri=req->req_info;
 
 			/* don't set the version number, for starters
 			 * the field is null and second, null is v0 
 			 * if (!ASN1_INTEGER_set(ci->version,0L)) goto end;
 			 */
-			if (!ASN1_INTEGER_set(ci->serialNumber,0L)) goto end;
-
-			/* steal ri->subject */
-			X509_NAME_free(ci->issuer);
-			ci->issuer=ri->subject;
-			ri->subject=NULL;
+			ASN1_INTEGER_set(X509_get_serialNumber(x509ss),0L);
 
+			X509_set_issuer_name(x509ss,
+				X509_REQ_get_subject_name(req));
 			X509_gmtime_adj(X509_get_notBefore(x509ss),0);
-			X509_gmtime_adj(X509_get_notAfter(x509ss),(long)60*60*24*days);
+			X509_gmtime_adj(X509_get_notAfter(x509ss),
+				(long)60*60*24*days);
+			X509_set_subject_name(x509ss,
+				X509_REQ_get_subject_name(req));
+			X509_set_pubkey(x509ss,X509_REQ_get_pubkey(req));
 
-			X509_NAME_free(ci->subject);
-			ci->subject=X509_NAME_dup(ci->issuer);
-
-			X509_PUBKEY_free(ci->key);
-			ci->key=ri->pubkey;
-			ri->pubkey=NULL;
-
-			pkey.type=EVP_PKEY_RSA;
-			pkey.pkey.rsa=rsa;
-			if (!(i=X509_sign(x509ss,&pkey,EVP_md5())))
+			if (!(i=X509_sign(x509ss,pkey,digest)))
 				goto end;
 			}
 		else
 			{
-			EVP_PKEY pkey;
-
-			pkey.type=EVP_PKEY_RSA;
-			pkey.pkey.rsa=rsa;
-
-			if (!(i=X509_REQ_sign(req,&pkey,EVP_md5())))
+			if (!(i=X509_REQ_sign(req,pkey,digest)))
 				goto end;
 			}
 		}
 
 	if (verify && !x509)
 		{
-		EVP_PKEY p;
-		EVP_PKEY *pkey=NULL;
+		int tmp=0;
 
-		if (rsa == NULL)
+		if (pkey == NULL)
 			{
 			pkey=X509_REQ_get_pubkey(req);
-			if (pkey == NULL)
-				{
-				ERR_print_errors(bio_err);
-				goto end;
-				}
-			}
-		else
-			{
-			p.type=EVP_PKEY_RSA;
-			p.pkey.rsa=rsa;
-			pkey= &p;
+			tmp=1;
+			if (pkey == NULL) goto end;
 			}
 
 		i=X509_REQ_verify(req,pkey);
-		if (rsa == NULL)
-			EVP_PKEY_free(pkey);
+		if (tmp) pkey=NULL;
+
 		if (i < 0)
 			{
-			ERR_print_errors(bio_err);
 			goto end;
 			}
 		else if (i == 0)
 			{
-			ERR_print_errors(bio_err);
 			BIO_printf(bio_err,"verify failure\n");
 			}
 		else /* if (i > 0) */
@@ -610,31 +670,23 @@ loop:
 
 	if (modulus)
 		{
-		EVP_PKEY *pkey;
+		EVP_PKEY *pubkey;
 
 		if (x509)
-			pkey=X509_get_pubkey(x509ss);
+			pubkey=X509_get_pubkey(x509ss);
 		else
-			pkey=X509_REQ_get_pubkey(req);
-		if (pkey == NULL)
+			pubkey=X509_REQ_get_pubkey(req);
+		if (pubkey == NULL)
 			{
 			fprintf(stdout,"Modulus=unavailable\n");
-			ERR_print_errors(bio_err);
 			goto end; 
 			}
 		fprintf(stdout,"Modulus=");
-		switch (pkey->type)
-			{
-		case EVP_PKEY_RSA:
-		case EVP_PKEY_RSA2:
-			BN_print(out,pkey->pkey.rsa->n);
-			break;
-		default:
+		if (pubkey->type == EVP_PKEY_RSA)
+			BN_print(out,pubkey->pkey.rsa->n);
+		else
 			fprintf(stdout,"Wrong Algorithm type");
-			break;
-			}
 		fprintf(stdout,"\n");
-		EVP_PKEY_free(pkey);
 		}
 
 	if (!noout && !x509)
@@ -650,7 +702,6 @@ loop:
 		if (!i)
 			{
 			BIO_printf(bio_err,"unable to write X509 request\n");
-			ERR_print_errors(bio_err);
 			goto end;
 			}
 		}
@@ -667,30 +718,32 @@ loop:
 		if (!i)
 			{
 			BIO_printf(bio_err,"unable to write X509 certificate\n");
-			ERR_print_errors(bio_err);
 			goto end;
 			}
 		}
 	ex=0;
 end:
+	if (ex)
+		{
+		ERR_print_errors(bio_err);
+		}
 	if ((req_conf != NULL) && (req_conf != config)) CONF_free(req_conf);
 	if (in != NULL) BIO_free(in);
 	if (out != NULL) BIO_free(out);
-	if (rsa != NULL) RSA_free(rsa);
+	if (pkey != NULL) EVP_PKEY_free(pkey);
 	if (req != NULL) X509_REQ_free(req);
 	if (x509ss != NULL) X509_free(x509ss);
+	if (dsa_params != NULL) DSA_free(dsa_params);
 	EXIT(ex);
 	}
 
-static int make_REQ(req,rsa,attribs)
+static int make_REQ(req,pkey,attribs)
 X509_REQ *req;
-RSA *rsa;
+EVP_PKEY *pkey;
 int attribs;
 	{
-	X509_ALGOR *a;
-	int ret=0,i;
-	unsigned char *s,*p;
-	ASN1_OBJECT *obj;
+	int ret=0,i,j;
+	unsigned char *p;
 	X509_REQ_INFO *ri;
 	char buf[100];
 	int nid,min,max;
@@ -727,7 +780,7 @@ int attribs;
 
 	ri=req->req_info;
 
-	BIO_printf(bio_err,"You are about to be asked to enter information that will be incorperated\n");
+	BIO_printf(bio_err,"You are about to be asked to enter information that will be incorporated\n");
 	BIO_printf(bio_err,"into your certificate request.\n");
 	BIO_printf(bio_err,"What you are about to enter is what is called a Distinguished Name or a DN.\n");
 	BIO_printf(bio_err,"There are quite a few fields but you can leave some blank\n");
@@ -747,29 +800,39 @@ start:		for (;;)
 			if ((int)sk_num(sk) <= i) break;
 
 			v=(CONF_VALUE *)sk_value(sk,i);
+			p=NULL;
 			type=v->name;
+			for (j=0; type[j] != '\0'; j++)
+				{
+				if (	(type[j] == ':') ||
+					(type[j] == ',') ||
+					(type[j] == '.'))
+					p= (unsigned char *)&(type[j+1]);
+				}
+			if (p != NULL)
+				type=(char *)p;
 			if ((nid=OBJ_txt2nid(type)) == NID_undef)
 				goto start;
 
-			sprintf(buf,"%s_default",type);
+			sprintf(buf,"%s_default",v->name);
 			if ((def=CONF_get_string(req_conf,tmp,buf)) == NULL)
 				def="";
 				
-			sprintf(buf,"%s_value",type);
+			sprintf(buf,"%s_value",v->name);
 			if ((value=CONF_get_string(req_conf,tmp,buf)) == NULL)
 				value=NULL;
 
-			sprintf(buf,"%s_min",type);
+			sprintf(buf,"%s_min",v->name);
 			min=(int)CONF_get_number(req_conf,tmp,buf);
 
-			sprintf(buf,"%s_max",type);
+			sprintf(buf,"%s_max",v->name);
 			max=(int)CONF_get_number(req_conf,tmp,buf);
 
 			if (!add_DN_object(ri->subject,v->value,def,value,nid,
 				min,max))
 				goto err;
 			}
-		if (sk_num(ri->subject) == 0)
+		if (sk_num(ri->subject->entries) == 0)
 			{
 			BIO_printf(bio_err,"error, no objects specified in config file\n");
 			goto err;
@@ -823,36 +886,7 @@ start2:			for (;;)
 		goto err;
 		}
 
-	/* copy public key and make algorithm*/
-	a=ri->pubkey->algor;
-
-	/* set the algorithm */
-	obj=OBJ_nid2obj(NID_rsaEncryption);
-	if (obj == NULL) goto err;
-	ASN1_OBJECT_free(a->algorithm);
-	a->algorithm=obj;
-
-	/* set the parameter */
-	if ((a->parameter == NULL) || (a->parameter->type != V_ASN1_NULL))
-		{
-		ASN1_TYPE_free(a->parameter);
-		a->parameter=ASN1_TYPE_new();
-		a->parameter->type=V_ASN1_NULL;
-		}
-
-	/* The public key */
-	i=i2d_RSAPublicKey(rsa,NULL);
-	if ((s=(unsigned char *)Malloc((unsigned int)i+1)) == NULL)
-		{
-		BIO_printf(bio_err,"Malloc failure\n");
-		goto err;
-		}
-	p=s;
-	i2d_RSAPublicKey(rsa,&p);
-	ri->pubkey->public_key->length=i;
-	if (ri->pubkey->public_key->data != NULL)
-		Free(ri->pubkey->public_key->data);
-	ri->pubkey->public_key->data=s;
+	X509_REQ_set_pubkey(req,pkey);
 
 	ret=1;
 err:
@@ -868,11 +902,10 @@ int nid;
 int min;
 int max;
 	{
-	int i,j,z;
+	int i,j,ret=0;
 	X509_NAME_ENTRY *ne=NULL;
 	MS_STATIC char buf[1024];
 
-start:
 	BIO_printf(bio_err,"%s [%s]:",text,def);
 	BIO_flush(bio_err);
 	if (value != NULL)
@@ -905,43 +938,19 @@ start:
 		}
 	buf[--i]='\0';
 
-	/* add object plus value */
-	if (sk_num(n) == 0)
-		j=0;
-	else
-		j=((X509_NAME_ENTRY *)(sk_value(n,sk_num(n)-1)))->set+1;
+	j=ASN1_PRINTABLE_type((unsigned char *)buf,-1);
+	if (req_fix_data(nid,&j,i,min,max) == 0)
+		goto err;
+	if ((ne=X509_NAME_ENTRY_create_by_NID(NULL,nid,j,(unsigned char *)buf,
+		strlen(buf)))
+		== NULL) goto err;
+	if (!X509_NAME_add_entry(n,ne,X509_NAME_entry_count(n),0))
+		goto err;
 
-	if (ne == NULL)
-		{
-		if ((ne=X509_NAME_ENTRY_new()) == NULL)
-			goto err;
-		}
-
-	ne->object=OBJ_nid2obj(nid);
-	ne->set=j;
-
-	ne->value->type=ASN1_PRINTABLE_type((unsigned char *)buf);
-
-	z=req_fix_data(nid,&ne->value->type,i,min,max);
-	if (z == 0)
-		{
-		if (value == NULL)
-			goto start;
-		else	goto err;
-		}
-
-	ne->value->length=i;
-	if (ne->value->data != NULL)
-		Free(ne->value->data);
-	ne->value->data=(unsigned char *)Malloc(i+1);
-	if (ne->value->data == NULL)
-		{ BIO_printf(bio_err,"Malloc failure\n"); goto err; }
-	memcpy(ne->value->data,buf,i+1);
-	if (!sk_push(n,(char *)ne)) goto err;
-	return(1);
+	ret=1;
 err:
 	if (ne != NULL) X509_NAME_ENTRY_free(ne);
-	return(0);
+	return(ret);
 	}
 
 static int add_attribute_object(n,text,def,value,nid,min,max)
@@ -954,9 +963,10 @@ int min;
 int max;
 	{
 	int i,z;
-	X509_ATTRIBUTE *ae=NULL;
+	X509_ATTRIBUTE *xa=NULL;
 	static char buf[1024];
 	ASN1_BIT_STRING *bs=NULL;
+	ASN1_TYPE *at=NULL;
 
 start:
 	BIO_printf(bio_err,"%s [%s]:",text,def);
@@ -992,21 +1002,18 @@ start:
 	buf[--i]='\0';
 
 	/* add object plus value */
-	if (ae == NULL)
-		{
-		if ((ae=X509_ATTRIBUTE_new()) == NULL)
-			goto err;
-		if ((ae->value.single=ASN1_TYPE_new()) == NULL)
-			goto err;
-		}
+	if ((xa=X509_ATTRIBUTE_new()) == NULL)
+		goto err;
+	if ((xa->value.set=sk_new_null()) == NULL)
+		goto err;
+	xa->set=1;
 
-	ae->object=OBJ_nid2obj(nid);
-	ae->nid=nid;
-	ae->set=0;
+	if (xa->object != NULL) ASN1_OBJECT_free(xa->object);
+	xa->object=OBJ_nid2obj(nid);
 
 	if ((bs=ASN1_BIT_STRING_new()) == NULL) goto err;
 
-	bs->type=ASN1_PRINTABLE_type((unsigned char *)buf);
+	bs->type=ASN1_PRINTABLE_type((unsigned char *)buf,-1);
 
 	z=req_fix_data(nid,&bs->type,i,min,max);
 	if (z == 0)
@@ -1016,21 +1023,23 @@ start:
 		else	goto err;
 		}
 
-	bs->length=i;
-	if (bs->data != NULL) Free(bs->data);
-	bs->data=(unsigned char *)Malloc(i+1);
-	if (bs->data == NULL)
+	if (!ASN1_STRING_set(bs,(unsigned char *)buf,i+1))
 		{ BIO_printf(bio_err,"Malloc failure\n"); goto err; }
-	memcpy(bs->data,buf,i+1);
 
-	ae->value.single->type=bs->type;
-	ae->value.single->value.ptr=(char *)bs;
+	if ((at=ASN1_TYPE_new()) == NULL)
+		{ BIO_printf(bio_err,"Malloc failure\n"); goto err; }
+
+	ASN1_TYPE_set(at,bs->type,(char *)bs);
+	sk_push(xa->value.set,(char *)at);
 	bs=NULL;
+	at=NULL;
+	/* only one item per attribute */
 
-	if (!sk_push(n,(char *)ae)) goto err;
+	if (!sk_push(n,(char *)xa)) goto err;
 	return(1);
 err:
-	if (ae != NULL) X509_ATTRIBUTE_free(ae);
+	if (xa != NULL) X509_ATTRIBUTE_free(xa);
+	if (at != NULL) ASN1_TYPE_free(at);
 	if (bs != NULL) ASN1_BIT_STRING_free(bs);
 	return(0);
 	}
