@@ -1,9 +1,9 @@
 /* apps/x509.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -59,13 +59,15 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef WIN16
+#define APPS_WIN16
+#endif
 #include "apps.h"
-#include "buffer.h"
+#include "bio.h"
 #include "asn1.h"
 #include "err.h"
 #include "bn.h"
-#include "rsa.h"
-#include "envelope.h"
+#include "evp.h"
 #include "x509.h"
 #include "objects.h"
 #include "pem.h"
@@ -88,7 +90,7 @@ static char *x509_usage[]={
 "usage: x509 args\n",
 " -inform arg     - input format - default PEM (one of DER, NET or PEM)\n",
 " -outform arg    - output format - default PEM (one of DER, NET or PEM\n",
-" -keyform arg    - rsa key format - default PEM\n",
+" -keyform arg    - private key format - default PEM\n",
 " -CAform arg     - CA format - default PEM\n",
 " -CAkeyform arg  - CA key format - default PEM\n",
 " -in arg         - input file - default stdin\n",
@@ -109,24 +111,26 @@ static char *x509_usage[]={
 " -x509toreq      - output a certification request object\n",
 " -req            - input is a certificate request, sign and output.\n",
 " -CA arg         - set the CA certificate, must be PEM format.\n",
-" -CAkey arg      - set the CA RSA key, must be PEM format\n",
+" -CAkey arg      - set the CA key, must be PEM format\n",
 "                   missing, it is asssumed to be in the CA file.\n",
 " -CAcreateserial - create serial number file if it does not exist\n",
 " -CAserial       - serial file\n",
 " -text           - print the certitificate in text form\n",
+" -C              - print out C code forms\n",
+" -md2/-md5/-sha1/-mdc2 - digest to do an RSA sign with\n",
 NULL
 };
 
 #ifndef NOPROTO
-static int MS_CALLBACK callb(int ok, X509 *xs, X509 *xi, int depth, int error);
-static RSA *load_key(char *file, int format);
+static int MS_CALLBACK callb(int ok, X509_STORE_CTX *ctx);
+static EVP_PKEY *load_key(char *file, int format);
 static X509 *load_cert(char *file, int format);
-static int sign (X509 *x, EVP_PKEY *pkey,int days);
-static int x509_certify (CERTIFICATE_CTX *ctx,char *CAfile, X509 *x,
+static int sign (X509 *x, EVP_PKEY *pkey,int days,EVP_MD *digest);
+static int x509_certify (X509_STORE *ctx,char *CAfile, EVP_MD *digest,X509 *x,
 	X509 *xca, EVP_PKEY *pkey,char *serial, int create, int days);
 #else
 static int MS_CALLBACK callb();
-static RSA *load_key();
+static EVP_PKEY *load_key();
 static X509 *load_cert();
 static int sign ();
 static int x509_certify ();
@@ -141,32 +145,31 @@ char **argv;
 	int ret=1;
 	X509_REQ *req=NULL;
 	X509 *x=NULL,*xca=NULL;
-	RSA *rsa=NULL,*CArsa=NULL;
+	EVP_PKEY *Upkey=NULL,*CApkey=NULL;
 	int i,num,badops=0;
 	BIO *out=NULL;
 	BIO *STDout=NULL;
 	int informat,outformat,keyformat,CAformat,CAkeyformat;
 	char *infile=NULL,*outfile=NULL,*keyfile=NULL,*CAfile=NULL;
-	char *CAkeyfile=NULL,*str=NULL,*CAserial=NULL;
+	char *CAkeyfile=NULL,*CAserial=NULL;
 	int text=0,serial=0,hash=0,subject=0,issuer=0,startdate=0,enddate=0;
 	int noout=0,sign_flag=0,CA_flag=0,CA_createserial=0;
+	int C=0;
 	int x509req=0,days=DEF_DAYS,modulus=0;
 	char **pp;
-	CERTIFICATE_CTX *ctx=NULL;
+	X509_STORE *ctx=NULL;
 	X509_REQ *rq=NULL;
-	RSA *r=NULL;
 	int fingerprint=0;
+	char buf[256];
+	EVP_MD *md_alg,*digest=EVP_md5();
 
 	reqfile=0;
 
 	apps_startup();
 
 	if (bio_err == NULL)
-		if ((bio_err=BIO_new(BIO_s_file())) != NULL)
-			BIO_set_fp(bio_err,stderr,BIO_NOCLOSE);
-
-	if ((STDout=BIO_new(BIO_s_file())) != NULL)
-		BIO_set_fp(STDout,stdout,BIO_NOCLOSE);
+		bio_err=BIO_new_fp(stderr,BIO_NOCLOSE);
+	STDout=BIO_new_fp(stdout,BIO_NOCLOSE);
 
 	informat=FORMAT_PEM;
 	outformat=FORMAT_PEM;
@@ -174,8 +177,10 @@ char **argv;
 	CAformat=FORMAT_PEM;
 	CAkeyformat=FORMAT_PEM;
 
-	ctx=CERTIFICATE_CTX_new();
+	ctx=X509_STORE_new();
 	if (ctx == NULL) goto end;
+	X509_STORE_set_verify_cb_func(ctx,callb);
+
 	argc--;
 	argv++;
 	num=0;
@@ -250,6 +255,8 @@ char **argv;
 			if (--argc < 1) goto bad;
 			CAserial= *(++argv);
 			}
+		else if (strcmp(*argv,"-C") == 0)
+			C= ++num;
 		else if (strcmp(*argv,"-serial") == 0)
 			serial= ++num;
 		else if (strcmp(*argv,"-modulus") == 0)
@@ -279,6 +286,11 @@ char **argv;
 			noout= ++num;
 		else if (strcmp(*argv,"-CAcreateserial") == 0)
 			CA_createserial= ++num;
+		else if ((md_alg=EVP_get_digestbyname(&((*argv)[1]))) != NULL)
+			{
+			/* ok */
+			digest=md_alg;
+			}
 		else
 			{
 			BIO_printf(bio_err,"unknown option %s\n",*argv);
@@ -299,7 +311,7 @@ bad:
 
 	ERR_load_crypto_strings();
 
-	if (!X509_set_default_verify_paths(ctx))
+	if (!X509_STORE_set_default_paths(ctx))
 		{
 		ERR_print_errors(bio_err);
 		goto end;
@@ -316,7 +328,6 @@ bad:
 	if (reqfile)
 		{
 		EVP_PKEY *pkey;
-		char *c;
 		X509_CINF *ci;
 		BIO *in;
 
@@ -362,7 +373,6 @@ bad:
 	                goto end;
 	                }
 		i=X509_REQ_verify(req,pkey);
-		EVP_PKEY_free(pkey);
 		if (i < 0)
 			{
 			BIO_printf(bio_err,"Signature verification error\n");
@@ -377,9 +387,8 @@ bad:
 		else
 			BIO_printf(bio_err,"Signature ok\n");
 		
-		c=X509_NAME_oneline(req->req_info->subject);
-		BIO_printf(bio_err,"subject=%s\n",c);
-		Free(c);
+		X509_NAME_oneline(req->req_info->subject,buf,256);
+		BIO_printf(bio_err,"subject=%s\n",buf);
 
 		if ((x=X509_new()) == NULL) goto end;
 		ci=x->cert_info;
@@ -407,6 +416,9 @@ bad:
 
 	if (!noout || text)
 		{
+		OBJ_create_and_add_object("2.99999.3",
+			"SET.ex3","SET x509v3 extension 3");
+
 		out=BIO_new(BIO_s_file());
 		if (out == NULL)
 			{
@@ -431,41 +443,30 @@ bad:
 			{
 			if (issuer == i)
 				{
-				str=X509_NAME_oneline(X509_get_issuer_name(x));
-				if (str == NULL)
-					{
-					BIO_printf(bio_err,"unable to get issuer Name from certificate\n");
-					ERR_print_errors(bio_err);
-					goto end;
-					}
-				fprintf(stdout,"issuer= %s\n",str);
-				Free(str);
+				X509_NAME_oneline(X509_get_issuer_name(x),
+					buf,256);
+				fprintf(stdout,"issuer= %s\n",buf);
 				}
-
-			if (subject == i) 
+			else if (subject == i) 
 				{
-				str=X509_NAME_oneline(X509_get_subject_name(x));
-				if (str == NULL)
-					{
-					BIO_printf(bio_err,"unable to get subject Name from certificate\n");
-					ERR_print_errors(bio_err);
-					goto end;
-					}
-				fprintf(stdout,"subject=%s\n",str);
-				Free(str);
+				X509_NAME_oneline(X509_get_subject_name(x),
+					buf,256);
+				fprintf(stdout,"subject=%s\n",buf);
 				}
-			if (serial == i)
+			else if (serial == i)
 				{
 				fprintf(stdout,"serial=");
 				i2a_ASN1_INTEGER(STDout,x->cert_info->serialNumber);
 				fprintf(stdout,"\n");
 				}
-			if (hash == i)
+			else if (hash == i)
 				{
 				fprintf(stdout,"%08lx\n",
 					X509_subject_name_hash(x));
 				}
-			if (modulus == i)
+			else
+#ifndef NO_RSA
+				if (modulus == i)
 				{
 				EVP_PKEY *pkey;
 
@@ -477,36 +478,83 @@ bad:
 					goto end;
 					}
 				fprintf(stdout,"Modulus=");
-				switch (pkey->type)
-					{
-				case EVP_PKEY_RSA:
-				case EVP_PKEY_RSA2:
+				if (pkey->type == EVP_PKEY_RSA)
 					BN_print(STDout,pkey->pkey.rsa->n);
-					break;
-				default:
+				else
 					fprintf(stdout,"Wrong Algorithm type");
-					break;
-					}
 				fprintf(stdout,"\n");
-				EVP_PKEY_free(pkey);
 				}
-			if (text == i)
+			else
+#endif
+				if (C == i)
+				{
+				unsigned char *d;
+				char *m;
+				int y,z;
+
+				X509_NAME_oneline(X509_get_subject_name(x),
+					buf,256);
+				printf("/* subject:%s */\n",buf);
+				m=X509_NAME_oneline(
+					X509_get_issuer_name(x),buf,256);
+				printf("/* issuer :%s */\n",buf);
+
+				z=i2d_X509(x,NULL);
+				m=Malloc(z);
+
+				d=(unsigned char *)m;
+				z=i2d_X509_NAME(X509_get_subject_name(x),&d);
+				printf("unsigned char XXX_subject_name[%d]={\n",z);
+				d=(unsigned char *)m;
+				for (y=0; y<z; y++)
+					{
+					printf("0x%02X,",d[y]);
+					if ((y & 0x0f) == 0x0f) printf("\n");
+					}
+				if (y%16 != 0) printf("\n");
+				printf("};\n");
+
+				z=i2d_X509_PUBKEY(X509_get_X509_PUBKEY(x),&d);
+				printf("unsigned char XXX_public_key[%d]={\n",z);
+				d=(unsigned char *)m;
+				for (y=0; y<z; y++)
+					{
+					printf("0x%02X,",d[y]);
+					if ((y & 0x0f) == 0x0f) printf("\n");
+					}
+				if (y%16 != 0) printf("\n");
+				printf("};\n");
+
+				z=i2d_X509(x,&d);
+				printf("unsigned char XXX_certificate[%d]={\n",z);
+				d=(unsigned char *)m;
+				for (y=0; y<z; y++)
+					{
+					printf("0x%02X,",d[y]);
+					if ((y & 0x0f) == 0x0f) printf("\n");
+					}
+				if (y%16 != 0) printf("\n");
+				printf("};\n");
+
+				Free(m);
+				}
+			else if (text == i)
 				{
 				X509_print(out,x);
 				}
-			if (startdate == i)
+			else if (startdate == i)
 				{
 				BIO_puts(STDout,"notBefore=");
 				ASN1_UTCTIME_print(STDout,X509_get_notBefore(x));
 				BIO_puts(STDout,"\n");
 				}
-			if (enddate == i)
+			else if (enddate == i)
 				{
 				BIO_puts(STDout,"notAfter=");
 				ASN1_UTCTIME_print(STDout,X509_get_notAfter(x));
 				BIO_puts(STDout,"\n");
 				}
-			if (fingerprint == i)
+			else if (fingerprint == i)
 				{
 				int j;
 				unsigned int n;
@@ -517,7 +565,7 @@ bad:
 					BIO_printf(bio_err,"out of memory\n");
 					goto end;
 					}
-				fprintf(stdout,"Fingerprint=");
+				fprintf(stdout,"MD5 Fingerprint=");
 				for (j=0; j<(int)n; j++)
 					{
 					fprintf(stdout,"%02X%c",md[j],
@@ -527,39 +575,41 @@ bad:
 				}
 
 			/* should be in the library */
-			if ((sign_flag == i) && (x509req == 0))
+			else if ((sign_flag == i) && (x509req == 0))
 				{
-				EVP_PKEY pkey;
-
 				BIO_printf(bio_err,"Getting Private key\n");
-				if (rsa == NULL)
+				if (Upkey == NULL)
 					{
-					rsa=load_key(keyfile,keyformat);
-					if (rsa == NULL) goto end;
+					Upkey=load_key(keyfile,keyformat);
+					if (Upkey == NULL) goto end;
 					}
-				pkey.type=EVP_PKEY_RSA;
-				pkey.pkey.rsa=rsa;
-				if (!sign(x,&pkey,days)) goto end;
-				}
-			if (CA_flag == i)
-				{
-				EVP_PKEY pkey;
+#ifndef NO_DSA
+		                if (Upkey->type == EVP_PKEY_DSA)
+		                        digest=EVP_dss1();
+#endif
 
+				if (!sign(x,Upkey,days,digest)) goto end;
+				}
+			else if (CA_flag == i)
+				{
 				BIO_printf(bio_err,"Getting CA Private Key\n");
 				if (CAkeyfile != NULL)
 					{
-					CArsa=load_key(CAkeyfile,CAkeyformat);
-					if (CArsa == NULL) goto end;
+					CApkey=load_key(CAkeyfile,CAkeyformat);
+					if (CApkey == NULL) goto end;
 					}
-				pkey.type=EVP_PKEY_RSA;
-				pkey.pkey.rsa=CArsa;
-				if (!x509_certify(ctx,CAfile,x,xca,&pkey,
+#ifndef NO_DSA
+		                if (CApkey->type == EVP_PKEY_DSA)
+		                        digest=EVP_dss1();
+#endif
+				if (!x509_certify(ctx,CAfile,digest,x,xca,
+					CApkey,
 					CAserial,CA_createserial,days))
 					goto end;
 				}
-			if (x509req == i)
+			else if (x509req == i)
 				{
-				EVP_PKEY pkey;
+				EVP_PKEY *pk;
 
 				BIO_printf(bio_err,"Getting request Private Key\n");
 				if (keyfile == NULL)
@@ -569,15 +619,14 @@ bad:
 					}
 				else
 					{
-					r=load_key(keyfile,FORMAT_PEM);
-					if (r == NULL) goto end;
+					pk=load_key(keyfile,FORMAT_PEM);
+					if (pk == NULL) goto end;
 					}
 
 				BIO_printf(bio_err,"Generating certificate request\n");
 
-				pkey.type=EVP_PKEY_RSA;
-				pkey.pkey.rsa=r;
-				rq=(X509_REQ *)X509_to_X509_REQ(x,&pkey);
+				rq=X509_to_X509_REQ(x,pk);
+				EVP_PKEY_free(pk);
 				if (rq == NULL)
 					{
 					ERR_print_errors(bio_err);
@@ -628,22 +677,23 @@ bad:
 		}
 	ret=0;
 end:
+	OBJ_cleanup();
 	if (out != NULL) BIO_free(out);
 	if (STDout != NULL) BIO_free(STDout);
-	if (ctx != NULL) CERTIFICATE_CTX_free(ctx);
+	if (ctx != NULL) X509_STORE_free(ctx);
 	if (req != NULL) X509_REQ_free(req);
 	if (x != NULL) X509_free(x);
 	if (xca != NULL) X509_free(xca);
-	if (rsa != NULL) RSA_free(rsa);
-	if (CArsa != NULL) RSA_free(CArsa);
+	if (Upkey != NULL) EVP_PKEY_free(Upkey);
+	if (CApkey != NULL) EVP_PKEY_free(CApkey);
 	if (rq != NULL) X509_REQ_free(rq);
-	if (r != NULL) RSA_free(r);
 	EXIT(ret);
 	}
 
-static int x509_certify(ctx,CAfile, x, xca, pkey, serialfile, create,days)
-CERTIFICATE_CTX *ctx;
+static int x509_certify(ctx,CAfile,digest,x,xca,pkey,serialfile,create,days)
+X509_STORE *ctx;
 char *CAfile;
+EVP_MD *digest;
 X509 *x;
 X509 *xca;
 EVP_PKEY *pkey;
@@ -653,10 +703,16 @@ int days;
 	{
 	int ret=0;
 	BIO *io=NULL;
-	MS_STATIC char *buf=NULL,buf2[1024];
+	MS_STATIC char buf2[1024];
+	char *buf=NULL,*p;
 	BIGNUM *serial=NULL;
 	ASN1_INTEGER *bs=NULL,bs2;
+	X509_STORE_CTX xsc;
+	EVP_PKEY *upkey;
 
+	EVP_PKEY_copy_parameters(X509_get_pubkey(xca),pkey);
+
+	X509_STORE_CTX_init(&xsc,ctx,x,NULL);
 	buf=(char *)Malloc(EVP_PKEY_size(pkey)*2+
 		((serialfile == NULL)
 			?(strlen(CAfile)+strlen(POSTFIX)+1)
@@ -665,6 +721,12 @@ int days;
 	if (serialfile == NULL)
 		{
 		strcpy(buf,CAfile);
+		for (p=buf; *p; p++)
+			if (*p == '.')
+				{
+				*p='\0';
+				break;
+				}
 		strcat(buf,POSTFIX);
 		}
 	else
@@ -732,14 +794,12 @@ int days;
 	BIO_free(io);
 	io=NULL;
 	
-	if (!X509_add_cert(ctx,x)) goto end;
-	/* Must up the references since X509_add_cert thinks it can keep
-	 * the cert for it's very own :-) */
-	x->references++;
+	if (!X509_STORE_add_cert(ctx,x)) goto end;
 
 	/* NOTE: this certificate can/should be self signed, unless it was
 	 * a certificate request in which case it is not. */
-	if (!reqfile && !X509_cert_verify(ctx,x,callb,NULL,NULL))
+	X509_STORE_CTX_set_cert(&xsc,x);
+	if (!reqfile && !X509_verify_cert(&xsc))
 		goto end;
 
 	if (!X509_set_issuer_name(x,X509_get_subject_name(xca))) goto end;
@@ -751,9 +811,20 @@ int days;
 	/* hardwired expired */
 	if (X509_gmtime_adj(X509_get_notAfter(x),(long)60*60*24*days) == NULL)
 		goto end;
-	if (!X509_sign(x,pkey,EVP_md5())) goto end;
+
+	/* don't save DSA parameters in child if parent has them. */
+	upkey=X509_get_pubkey(x);
+	if (!EVP_PKEY_missing_parameters(pkey))
+		{
+		EVP_PKEY_save_parameters(upkey,0);
+		/* Force a re-write */
+		X509_set_pubkey(x,upkey);
+		}
+
+	if (!X509_sign(x,pkey,digest)) goto end;
 	ret=1;
 end:
+	X509_STORE_CTX_cleanup(&xsc);
 	if (!ret)
 		ERR_print_errors(bio_err);
 	if (buf != NULL) Free(buf);
@@ -763,42 +834,47 @@ end:
 	return(ret);
 	}
 
-static int MS_CALLBACK callb(ok, xs, xi, depth, error)
+static int MS_CALLBACK callb(ok, ctx)
 int ok;
-X509 *xs;
-X509 *xi;
-int depth;
-int error;
+X509_STORE_CTX *ctx;
 	{
-	/* it is ok to use a self signed certificate */
-	if ((!ok) && (error == VERIFY_ERR_DEPTH_ZERO_SELF_SIGNED_CERT))
+	char buf[256];
+	int err;
+	X509 *err_cert;
+
+	/* it is ok to use a self signed certificate
+	 * This case will catch both the initial ok == 0 and the
+	 * final ok == 1 calls to this function */
+	err=X509_STORE_CTX_get_error(ctx);
+	if (err == X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT)
 		return(1);
 
-	/* BAD we should have gotten an error :-) */
+	/* BAD we should have gotten an error.  Normally if everything
+	 * worked X509_STORE_CTX_get_error(ctx) will still be set to
+	 * DEPTH_ZERO_SELF_.... */
 	if (ok)
+		{
 		printf("error with certificate to be certified - should be self signed\n");
+		return(0);
+		}
 	else
 		{
-		char *s;
-
-		s=X509_NAME_oneline(X509_get_subject_name(xs));
-		printf("%s\n",s);
-		Free(s);
+		err_cert=X509_STORE_CTX_get_current_cert(ctx);
+		X509_NAME_oneline(X509_get_subject_name(err_cert),buf,256);
+		printf("%s\n",buf);
 		printf("error with certificate - error %d at depth %d\n%s\n",
-			error,depth,X509_cert_verify_error_string(error));
+			err,X509_STORE_CTX_get_error_depth(ctx),
+			X509_verify_cert_error_string(err));
+		return(1);
 		}
-#ifdef LINT
-	xi=xs; xs=xi;
-#endif
-	return(0);
 	}
 
-static RSA *load_key(file, format)
+static EVP_PKEY *load_key(file, format)
 char *file;
 int format;
 	{
 	BIO *key=NULL;
-	RSA *rsa=NULL;
+	EVP_PKEY *pkey=NULL;
 
 	if (file == NULL)
 		{
@@ -816,10 +892,26 @@ int format;
 		perror(file);
 		goto end;
 		}
+#ifndef NO_RSA
 	if	(format == FORMAT_ASN1)
+		{
+		RSA *rsa;
+
 		rsa=d2i_RSAPrivateKey_bio(key,NULL);
-	else if (format == FORMAT_PEM)
-		rsa=PEM_read_bio_RSAPrivateKey(key,NULL,NULL);
+		if (rsa != NULL)
+			{
+			if ((pkey=EVP_PKEY_new()) != NULL)
+				EVP_PKEY_assign_RSA(pkey,rsa);
+			else
+				RSA_free(rsa);
+			}
+		}
+	else
+#endif
+		if (format == FORMAT_PEM)
+		{
+		pkey=PEM_read_bio_PrivateKey(key,NULL,NULL);
+		}
 	else
 		{
 		BIO_printf(bio_err,"bad input format specified for key\n");
@@ -827,9 +919,9 @@ int format;
 		}
 end:
 	if (key != NULL) BIO_free(key);
-	if (rsa == NULL)
+	if (pkey == NULL)
 		BIO_printf(bio_err,"unable to load Private Key\n");
-	return(rsa);
+	return(pkey);
 	}
 
 static X509 *load_cert(file, format)
@@ -921,11 +1013,16 @@ end:
 	}
 
 /* self sign */
-static int sign(x, pkey, days)
+static int sign(x, pkey, days, digest)
 X509 *x;
 EVP_PKEY *pkey;
 int days;
+EVP_MD *digest;
 	{
+
+	EVP_PKEY_copy_parameters(X509_get_pubkey(x),pkey);
+	EVP_PKEY_save_parameters(X509_get_pubkey(x),1);
+
 	if (!X509_set_issuer_name(x,X509_get_subject_name(x))) goto err;
 	if (X509_gmtime_adj(X509_get_notBefore(x),0) == NULL) goto err;
 
@@ -937,7 +1034,7 @@ int days;
 		goto err;
 
 	if (!X509_set_pubkey(x,pkey)) goto err;
-	if (!X509_sign(x,pkey,EVP_md5())) goto err;
+	if (!X509_sign(x,pkey,digest)) goto err;
 	return(1);
 err:
 	ERR_print_errors(bio_err);
