@@ -1,9 +1,9 @@
 /* crypto/pem/pem_lib.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -60,18 +60,15 @@
 #include "cryptlib.h"
 #include "buffer.h"
 #include "objects.h"
-#include "rsa.h"
-#include "envelope.h"
+#include "evp.h"
 #include "rand.h"
 #include "x509.h"
 #include "pem.h"
-#ifdef NO_DES
-#include <des.h>
-#else
+#ifndef NO_DES
 #include "des.h"
 #endif
 
-char *PEM_version="PEM part of SSLeay 0.6.6 14-Jan-1997";
+char *PEM_version="PEM part of SSLeay 0.8.1 19-Jul-1997";
 
 #define MIN_LENGTH	4
 
@@ -80,13 +77,9 @@ char *PEM_version="PEM part of SSLeay 0.6.6 14-Jan-1997";
  */
 
 #ifndef NOPROTO
-static void PEM_proc_type(char *buf, int type);
-static void PEM_dek_info(char *buf, char *type, int len, char *str);
 static int def_callback(char *buf, int num, int w);
 static int load_iv(unsigned char **fromp,unsigned char *to, int num);
 #else
-static void PEM_proc_type();
-static void PEM_dek_info();
 static int def_callback();
 static int load_iv();
 #endif
@@ -130,7 +123,7 @@ int w;
 #endif
 	}
 
-static void PEM_proc_type(buf, type)
+void PEM_proc_type(buf, type)
 char *buf;
 int type;
 	{
@@ -150,7 +143,7 @@ int type;
 	strcat(buf,"\n");
 	}
 
-static void PEM_dek_info(buf, type, len, str)
+void PEM_dek_info(buf, type, len, str)
 char *buf;
 char *type;
 int len;
@@ -295,7 +288,7 @@ int (*callback)();
 	
 	if (enc != NULL)
 		{
-		objstr=OBJ_nid2sn(enc->type);
+		objstr=OBJ_nid2sn(EVP_CIPHER_nid(enc));
 		if (objstr == NULL)
 			{
 			PEMerr(PEM_F_PEM_ASN1_WRITE_BIO,PEM_R_UNSUPPORTED_CIPHER);
@@ -396,7 +389,9 @@ int (*callback)();
 	EVP_DecryptInit(&ctx,cipher->cipher,key,&(cipher->iv[0]));
 	EVP_DecryptUpdate(&ctx,data,&i,data,j);
 	o=EVP_DecryptFinal(&ctx,&(data[i]),&j);
-	memset((char *)&ctx,0,sizeof(ctx));
+	EVP_CIPHER_CTX_cleanup(&ctx);
+	memset((char *)buf,0,sizeof(buf));
+	memset((char *)key,0,sizeof(key));
 	j+=i;
 	if (!o)
 		{
@@ -445,23 +440,12 @@ EVP_CIPHER_INFO *cipher;
 		}
 	*header='\0';
 	o=OBJ_sn2nid(p);
+	cipher->cipher=enc=EVP_get_cipherbyname(p);
 	*header=c;
 	header++;
 
-	switch (o)
+	if (enc == NULL)
 		{
-	case NID_des_cbc:
-		cipher->cipher=enc=EVP_des_cbc();
-		break;
-	case NID_des_ede3_cbc:
-		cipher->cipher=EVP_des_ede3_cbc();
-		break;
-#ifndef NO_IDEA
-	case NID_idea_cbc:
-		cipher->cipher=EVP_idea_cbc();
-		break;
-#endif
-	default:
 		PEMerr(PEM_F_PEM_GET_EVP_CIPHER_INFO,PEM_R_UNSUPPORTED_ENCRYPTION);
 		return(0);
 		}
