@@ -1,9 +1,9 @@
 /* crypto/objects/objects.h */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -69,7 +69,7 @@ extern "C" {
 
 #define SN_Algorithm			"Algorithm"
 #define LN_algorithm			"algorithm"
-#define NID				38
+#define NID_algorithm			38
 #define OBJ_algorithm			1L,3L,14L,3L,2L
 
 #define LN_rsadsi			"rsadsi"
@@ -99,15 +99,15 @@ extern "C" {
 #define NID_rsaEncryption		6
 #define OBJ_rsaEncryption		OBJ_pkcs,1L,1L
 
-#define SN_md2withRSAEncryption		"RSA-MD2"
-#define LN_md2withRSAEncryption		"md2withRSAEncryption"
-#define NID_md2withRSAEncryption	7
-#define OBJ_md2withRSAEncryption	OBJ_pkcs,1L,2L
+#define SN_md2WithRSAEncryption		"RSA-MD2"
+#define LN_md2WithRSAEncryption		"md2WithRSAEncryption"
+#define NID_md2WithRSAEncryption	7
+#define OBJ_md2WithRSAEncryption	OBJ_pkcs,1L,2L
 
-#define SN_md5withRSAEncryption		"RSA-MD5"
-#define LN_md5withRSAEncryption		"md5withRSAEncryption"
-#define NID_md5withRSAEncryption	8
-#define OBJ_md5withRSAEncryption	OBJ_pkcs,1L,4L
+#define SN_md5WithRSAEncryption		"RSA-MD5"
+#define LN_md5WithRSAEncryption		"md5WithRSAEncryption"
+#define NID_md5WithRSAEncryption	8
+#define OBJ_md5WithRSAEncryption	OBJ_pkcs,1L,4L
 
 #define LN_pbeWithMD2AndDES_CBC		"pbeWithMD2AndDES-CBC"
 #define NID_pbeWithMD2AndDES_CBC	9
@@ -142,7 +142,8 @@ extern "C" {
 
 /* Postal Address? PA */
 
-#define SN_stateOrProvinceName		"SP"
+/* should be "ST" (rfc1327) but MS uses 'S' */
+#define SN_stateOrProvinceName		"ST"
 #define LN_stateOrProvinceName		"stateOrProvinceName"
 #define NID_stateOrProvinceName		16
 #define OBJ_stateOrProvinceName		OBJ_X509,8L
@@ -203,32 +204,35 @@ extern "C" {
 #define NID_des_ecb			29
 #define OBJ_des_ecb			OBJ_algorithm,6L
 
-#define SN_des_cfb			"DES-CFB"
-#define LN_des_cfb			"des-cfb"
-#define NID_des_cfb			30
-#define OBJ_des_cfb			OBJ_algorithm,9L
+#define SN_des_cfb64			"DES-CFB"
+#define LN_des_cfb64			"des-cfb"
+#define NID_des_cfb64			30
+/* IV + num */
+#define OBJ_des_cfb64			OBJ_algorithm,9L
 
 #define SN_des_cbc			"DES-CBC"
 #define LN_des_cbc			"des-cbc"
 #define NID_des_cbc			31
+/* IV */
 #define OBJ_des_cbc			OBJ_algorithm,7L
 
 #define SN_des_ede			"DES-EDE"
 #define LN_des_ede			"des-ede"
 #define NID_des_ede			32
+/* ?? */
+#define OBJ_des_ede			OBJ_algorithm,17L
 
 #define SN_des_ede3			"DES-EDE3"
 #define LN_des_ede3			"des-ede3"
 #define NID_des_ede3			33
-#define OBJ_des_ede3			OBJ_rsadsi,3L,17L
 
 #define SN_idea_cbc			"IDEA-CBC"
 #define LN_idea_cbc			"idea-cbc"
 #define NID_idea_cbc			34
 
-#define SN_idea_cfb			"IDEA-CFB"
-#define LN_idea_cfb			"idea-cfb"
-#define NID_idea_cfb			35
+#define SN_idea_cfb64			"IDEA-CFB"
+#define LN_idea_cfb64			"idea-cfb"
+#define NID_idea_cfb64			35
 
 #define SN_idea_ecb			"IDEA-ECB"
 #define LN_idea_ecb			"idea-ecb"
@@ -243,13 +247,13 @@ extern "C" {
 #define LN_rc2_ecb			"rc2-ecb"
 #define NID_rc2_ecb			38
 
-#define SN_rc2_cfb			"RC2-CFB"
-#define LN_rc2_cfb			"rc2-cfb"
-#define NID_rc2_cfb			39
+#define SN_rc2_cfb64			"RC2-CFB"
+#define LN_rc2_cfb64			"rc2-cfb"
+#define NID_rc2_cfb64			39
 
-#define SN_rc2_ofb			"RC2-OFB"
-#define LN_rc2_ofb			"rc2-ofb"
-#define NID_rc2_ofb			40
+#define SN_rc2_ofb64			"RC2-OFB"
+#define LN_rc2_ofb64			"rc2-ofb"
+#define NID_rc2_ofb64			40
 
 #define SN_sha				"SHA"
 #define LN_sha				"sha"
@@ -259,7 +263,7 @@ extern "C" {
 #define SN_shaWithRSAEncryption		"RSA-SHA"
 #define LN_shaWithRSAEncryption		"shaWithRSAEncryption"
 #define NID_shaWithRSAEncryption	42
-#define OBJ_shawithRSAEncryption	OBJ_algorithm,15L
+#define OBJ_shaWithRSAEncryption	OBJ_algorithm,15L
 
 #define SN_des_ede_cbc			"DES-EDE-CBC"
 #define LN_des_ede_cbc			"des-ede-cbc"
@@ -270,14 +274,14 @@ extern "C" {
 #define NID_des_ede3_cbc		44
 #define OBJ_des_ede3_cbc		OBJ_rsadsi,3L,7L
 
-#define SN_des_ofb			"DES-OFB"
-#define LN_des_ofb			"des-ofb"
-#define NID_des_ofb			45
-#define OBJ_des_ofb			OBJ_algorithm,8L
+#define SN_des_ofb64			"DES-OFB"
+#define LN_des_ofb64			"des-ofb"
+#define NID_des_ofb64			45
+#define OBJ_des_ofb64			OBJ_algorithm,8L
 
-#define SN_idea_ofb			"IDEA-OFB"
-#define LN_idea_ofb			"idea-ofb"
-#define NID_idea_ofb			46
+#define SN_idea_ofb64			"IDEA-OFB"
+#define LN_idea_ofb64			"idea-ofb"
+#define NID_idea_ofb64			46
 
 #define LN_pkcs9			"pkcs9"
 #define NID_pkcs9			47
@@ -335,21 +339,21 @@ extern "C" {
 #define NID_netscape_data_type		59
 #define OBJ_netscape_data_type		OBJ_netscape,2L
 
-#define SN_des_ede_cfb			"DES-EDE-CFB"
-#define LN_des_ede_cfb			"des-ede-cfb"
-#define NID_des_ede_cfb			60
+#define SN_des_ede_cfb64		"DES-EDE-CFB"
+#define LN_des_ede_cfb64		"des-ede-cfb"
+#define NID_des_ede_cfb64		60
 
-#define SN_des_ede3_cfb			"DES-EDE3-CFB"
-#define LN_des_ede3_cfb			"des-ede3-cfb"
-#define NID_des_ede3_cfb		61
+#define SN_des_ede3_cfb64		"DES-EDE3-CFB"
+#define LN_des_ede3_cfb64		"des-ede3-cfb"
+#define NID_des_ede3_cfb64		61
 
-#define SN_des_ede_ofb			"DES-EDE-OFB"
-#define LN_des_ede_ofb			"des-ede-ofb"
-#define NID_des_ede_ofb			62
+#define SN_des_ede_ofb64		"DES-EDE-OFB"
+#define LN_des_ede_ofb64		"des-ede-ofb"
+#define NID_des_ede_ofb64		62
 
-#define SN_des_ede3_ofb			"DES-EDE3-OFB"
-#define LN_des_ede3_ofb			"des-ede3-ofb"
-#define NID_des_ede3_ofb		63
+#define SN_des_ede3_ofb64		"DES-EDE3-OFB"
+#define LN_des_ede3_ofb64		"des-ede3-ofb"
+#define NID_des_ede3_ofb64		63
 
 /* I'm not sure about the object ID */
 #define SN_sha1				"SHA1"
@@ -387,6 +391,8 @@ extern "C" {
 #define SN_dsaWithSHA1			"DSA-SHA1"
 #define LN_dsaWithSHA1			"dsaWithSHA1"
 #define NID_dsaWithSHA1			70
+/* Got this one from 'sdn706r20.pdf' which is actually an NSA document :-) */
+#define OBJ_dsaWithSHA1			OBJ_algorithm,27L
 
 #define SN_netscape_cert_type		"nsCertType"
 #define LN_netscape_cert_type		"Netscape Cert Type"
@@ -441,48 +447,76 @@ extern "C" {
 #define NID_ld_ce			81
 #define OBJ_ld_ce			2L,5L,29L
 
-#define SN_authority_key_identifier	"authorityKeyIdentifier"
-#define LN_authority_key_identifier	"X509v3 Authority Key Identifier"
-#define NID_authority_key_identifier	82
-#define OBJ_authority_key_identifier	OBJ_ld_ce,35L
-
 #define SN_subject_key_identifier	"subjectKeyIdentifier"
 #define LN_subject_key_identifier	"X509v3 Subject Key Identifier"
-#define NID_subject_key_identifier	83
+#define NID_subject_key_identifier	82
 #define OBJ_subject_key_identifier	OBJ_ld_ce,14L
 
 #define SN_key_usage			"keyUsage"
 #define LN_key_usage			"X509v3 Key Usage"
-#define NID_key_usage			84
+#define NID_key_usage			83
 #define OBJ_key_usage			OBJ_ld_ce,15L
 
 #define SN_private_key_usage_period	"privateKeyUsagePeriod"
 #define LN_private_key_usage_period	"X509v3 Private Key Usage Period"
-#define NID_private_key_usage_period	85
+#define NID_private_key_usage_period	84
 #define OBJ_private_key_usage_period	OBJ_ld_ce,16L
 
+#define SN_subject_alt_name		"subjectAltName"
+#define LN_subject_alt_name		"X509v3 Subject Alternative Name"
+#define NID_subject_alt_name		85
+#define OBJ_subject_alt_name		OBJ_ld_ce,17L
+
+#define SN_issuer_alt_name		"issuerAltName"
+#define LN_issuer_alt_name		"X509v3 Issuer Alternative Name"
+#define NID_issuer_alt_name		86
+#define OBJ_issuer_alt_name		OBJ_ld_ce,18L
+
+#define SN_basic_constraints		"basicConstraints"
+#define LN_basic_constraints		"X509v3 Basic Constraints"
+#define NID_basic_constraints		87
+#define OBJ_basic_constraints		OBJ_ld_ce,19L
+
+#define SN_crl_number			"crlNumber"
+#define LN_crl_number			"X509v3 CRL Number"
+#define NID_crl_number			88
+#define OBJ_crl_number			OBJ_ld_ce,20L
+
 #define SN_certificate_policies		"certificatePolicies"
 #define LN_certificate_policies		"X509v3 Certificate Policies"
-#define NID_certificate_policies	86
-#define OBJ_certificate_policies	OBJ_ld_ce,32
+#define NID_certificate_policies	89
+#define OBJ_certificate_policies	OBJ_ld_ce,32L
+
+#define SN_authority_key_identifier	"authorityKeyIdentifier"
+#define LN_authority_key_identifier	"X509v3 Authority Key Identifier"
+#define NID_authority_key_identifier	90
+#define OBJ_authority_key_identifier	OBJ_ld_ce,35L
 
 #define SN_bf_cbc			"BF-CBC"
 #define LN_bf_cbc			"bf-cbc"
-#define NID_bf_cbc			87
-#define OBJ_bf_cbc			OBJ_rsadsi,3L,2L
+#define NID_bf_cbc			91
 
 #define SN_bf_ecb			"BF-ECB"
 #define LN_bf_ecb			"bf-ecb"
-#define NID_bf_ecb			88
+#define NID_bf_ecb			92
 
-#define SN_bf_cfb			"BF-CFB"
-#define LN_bf_cfb			"bf-cfb"
-#define NID_bf_cfb			89
+#define SN_bf_cfb64			"BF-CFB"
+#define LN_bf_cfb64			"bf-cfb"
+#define NID_bf_cfb64			93
 
-#define SN_bf_ofb			"BF-OFB"
-#define LN_bf_ofb			"bf-ofb"
-#define NID_bf_ofb			90
+#define SN_bf_ofb64			"BF-OFB"
+#define LN_bf_ofb64			"bf-ofb"
+#define NID_bf_ofb64			94
 
+#define SN_mdc2				"MDC2"
+#define LN_mdc2				"mdc2"
+#define NID_mdc2			95
+#define OBJ_mdc2			2L,5L,8L,3L,101L
+
+#define SN_mdc2WithRSA			"RSA-MDC2"
+#define LN_mdc2WithRSA			"mdc2withRSA"
+#define NID_mdc2WithRSA			96
+#define OBJ_mdc2WithRSA			2L,5L,8L,3L,100L
 
 #include "asn1.h"
 
@@ -500,6 +534,13 @@ int		OBJ_cmp(ASN1_OBJECT *a,ASN1_OBJECT *b);
 char *		OBJ_bsearch(char *key,char *base,int num,int size,int (*cmp)());
 
 void		ERR_load_OBJ_strings(void );
+
+int		OBJ_new_nid(int num);
+int		OBJ_add_object(ASN1_OBJECT *obj);
+int		OBJ_create_and_add_object(char *oid,char *sn,char *ln);
+
+void		OBJ_cleanup(void );
+
 #else
 
 ASN1_OBJECT *	OBJ_dup();
@@ -515,21 +556,27 @@ char *		OBJ_bsearch();
 
 void		ERR_load_OBJ_strings();
 
+int		OBJ_new_nid();
+int		OBJ_add_object();
+int		OBJ_create_and_add_object();
+void		OBJ_cleanup();
+
 #endif
 
 /* BEGIN ERROR CODES */
 /* Error codes for the OBJ functions. */
 
 /* Function codes. */
-#define OBJ_F_OBJ_DUP					 100
-#define OBJ_F_OBJ_NID2LN				 101
-#define OBJ_F_OBJ_NID2OBJ				 102
-#define OBJ_F_OBJ_NID2SN				 103
+#define OBJ_F_OBJ_CREATE_AND_ADD_OBJECT			 100
+#define OBJ_F_OBJ_DUP					 101
+#define OBJ_F_OBJ_NID2LN				 102
+#define OBJ_F_OBJ_NID2OBJ				 103
+#define OBJ_F_OBJ_NID2SN				 104
 
 /* Reason codes. */
-#define OBJ_R_NID_IS_OUT_OF_RANGE			 100
+#define OBJ_R_MALLOC_FAILURE				 100
 #define OBJ_R_UNKNOWN_NID				 101
-
+ 
 #ifdef  __cplusplus
 }
 #endif
