@@ -1,9 +1,9 @@
 /* ssl/ssl_rsa.c */
-/* Copyright (C) 1995-1997 Eric Young (eay@mincom.oz.au)
+/* Copyright (C) 1995-1997 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
  * This package is an SSL implementation written
- * by Eric Young (eay@mincom.oz.au).
+ * by Eric Young (eay@cryptsoft.com).
  * The implementation was written so as to conform with Netscapes SSL.
  * 
  * This library is free for commercial and non-commercial use as long as
@@ -11,7 +11,7 @@
  * apply to all code found in this distribution, be it the RC4, RSA,
  * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
  * included with this distribution is covered by the same copyright terms
- * except that the holder is Tim Hudson (tjh@mincom.oz.au).
+ * except that the holder is Tim Hudson (tjh@cryptsoft.com).
  * 
  * Copyright remains Eric Young's, and as such any Copyright notices in
  * the code are not to be removed.
@@ -31,12 +31,12 @@
  * 3. All advertising materials mentioning features or use of this software
  *    must display the following acknowledgement:
  *    "This product includes cryptographic software written by
- *     Eric Young (eay@mincom.oz.au)"
+ *     Eric Young (eay@cryptsoft.com)"
  *    The word 'cryptographic' can be left out if the rouines from the library
  *    being used are not cryptographic related :-).
  * 4. If you include any Windows specific code (or a derivative thereof) from 
  *    the apps directory (application code) you must include an acknowledgement:
- *    "This product includes software written by Tim Hudson (tjh@mincom.oz.au)"
+ *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
  * 
  * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -57,100 +57,26 @@
  */
 
 #include <stdio.h>
-#include "buffer.h"
+#include "bio.h"
 #include "objects.h"
-#include "rsa.h"
+#include "evp.h"
 #include "x509.h"
 #include "pem.h"
 #include "ssl_locl.h"
-#include "ssl_rsa.h"
 
-int ssl_rsa_public_encrypt(c, len, from, to)
-CERT *c;
-int len;
-unsigned char *from;
-unsigned char *to;
-	{
-	RSA *rsa;
-	int i;
-
-	if ((c == NULL) || (c->publickey == NULL))
-		{
-		SSLerr(SSL_F_SSL_RSA_PUBLIC_ENCRYPT,SSL_R_NO_PUBLICKEY);
-		return(-1);
-		}
-	if (c->publickey->type != EVP_PKEY_RSA)
-		{
-		SSLerr(SSL_F_SSL_RSA_PUBLIC_ENCRYPT,SSL_R_PUBLIC_KEY_IS_NOT_RSA);
-		return(-1);
-		}
-
-	rsa=c->publickey->pkey.rsa;
-	/* we have the public key */
-#ifdef RSA_DEBUG
-	SSL_TRACE(SSL_ERR,"PUBLIC RSA INPUT %d %d\n",len,to);
-	for (j=0; j<len; j++) SSL_TRACE(SSL_ERR,"%02X ",from[j]);
-	SSL_TRACE(SSL_ERR,"\n");
-	for (j=0; j<64; j++) SSL_TRACE(SSL_ERR,"%02X ",to[j]);
-	SSL_TRACE(SSL_ERR,"\n");
+#ifndef NOPROTO
+static int ssl_set_cert(CERT *c, X509 *x509);
+static int ssl_set_pkey(CERT *c, EVP_PKEY *pkey);
+#else
+static int ssl_set_cert();
+static int ssl_set_pkey();
 #endif
-	i=RSA_public_encrypt(len,from,to,rsa);
-	if (i < 0)
-		{
-		SSLerr(SSL_F_SSL_RSA_PUBLIC_ENCRYPT,ERR_R_RSA_LIB);
-		}
-#ifdef RSA_DEBUG
-	SSL_TRACE(SSL_ERR,"PUBLIC RSA OUTPUT %d %08x\n",i,to);
-	for (j=0; j<i; j++) SSL_TRACE(SSL_ERR,"%02X ",to[j]);
-	SSL_TRACE(SSL_ERR,"\n");
-#endif
-	return(i);
-	}
-
-int ssl_rsa_private_decrypt(c, len, from, to)
-CERT *c;
-int len;
-unsigned char *from;
-unsigned char *to;
-	{
-	RSA *rsa;
-	int i;
-
-	if ((c == NULL) || (c->privatekey == NULL))
-		{
-		SSLerr(SSL_F_SSL_RSA_PRIVATE_DECRYPT,SSL_R_NO_PRIVATEKEY);
-		return(-1);
-		}
-	if (c->privatekey->type != EVP_PKEY_RSA)
-		{
-		SSLerr(SSL_F_SSL_RSA_PRIVATE_DECRYPT,SSL_R_PUBLIC_KEY_IS_NOT_RSA);
-		return(-1);
-		}
-	rsa=c->privatekey->pkey.rsa;
-
-	/* we have the public key */
-#ifdef RSA_DEBUG
-	SSL_TRACE(SSL_ERR,"PRIVATE RSA INPUT %d\n",len);
-	for (j=0; j<len; j++) SSL_TRACE(SSL_ERR,"%02X ",from[j]);
-	SSL_TRACE(SSL_ERR,"\n");
-#endif
-	i=RSA_private_decrypt(len,from,to,rsa);
-	if (i < 0)
-		SSLerr(SSL_F_SSL_RSA_PRIVATE_DECRYPT,ERR_R_RSA_LIB);
-#ifdef RSA_DEBUG
-	SSL_TRACE(SSL_ERR,"PRIVATE RSA OUTPUT %d\n",i);
-	for (j=0; j<i; j++) SSL_TRACE(SSL_ERR,"%02X ",to[j]);
-	SSL_TRACE(SSL_ERR,"\n");
-#endif
-	return(i);
-	}
 
 int SSL_use_certificate(ssl, x)
 SSL *ssl;
 X509 *x;
 	{
 	CERT *c;
-	EVP_PKEY *pkey;
 
 	if (x == NULL)
 		{
@@ -159,26 +85,18 @@ X509 *x;
 		}
 	if ((ssl->cert == NULL) || (ssl->cert == ssl->ctx->default_cert))
 		{
-		if (ssl->cert != NULL) ssl_cert_free(ssl->cert);
 		c=ssl_cert_new();
-		if (c == NULL) return(0);
+		if (c == NULL)
+			{
+			SSLerr(SSL_F_SSL_USE_CERTIFICATE,ERR_R_MALLOC_FAILURE);
+			return(0);
+			}
+		if (ssl->cert != NULL) ssl_cert_free(ssl->cert);
 		ssl->cert=c;
 		}
 	c=ssl->cert;
 
-	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE)) return(0);
-	if (c->x509 != NULL) X509_free(c->x509);
-	CRYPTO_add(&x->references,1,CRYPTO_LOCK_X509);
-	c->x509=x;
-	if (c->publickey != NULL) EVP_PKEY_free(c->publickey);
-	pkey=X509_get_pubkey(x);
-	if (pkey == NULL)
-		{
-		SSLerr(SSL_F_SSL_USE_CERTIFICATE,SSL_R_X509_LIB);
-		return(0);
-		}
-	c->publickey=pkey;
-	return(1);
+	return(ssl_set_cert(c,x));
 	}
 
 int SSL_use_certificate_file(ssl, file, type)
@@ -204,7 +122,7 @@ int type;
 
 	if (BIO_read_filename(in,file) <= 0)
 		{
-		SYSerr(ERR_F_FOPEN,errno);
+		SYSerr(SYS_F_FOPEN,errno);
 		SSLerr(SSL_F_SSL_USE_CERTIFICATE_FILE,ERR_R_SYS_LIB);
 		goto end;
 		}
@@ -231,7 +149,6 @@ int type;
 		}
 
 	ret=SSL_use_certificate(ssl,x);
-	ret=1;
 end:
 	if (x != NULL) X509_free(x);
 	if (in != NULL) BIO_free(in);
@@ -258,11 +175,14 @@ unsigned char *d;
 	return(ret);
 	}
 
+#ifndef NO_RSA
 int SSL_use_RSAPrivateKey(ssl, rsa)
 SSL *ssl;
 RSA *rsa;
 	{
 	CERT *c;
+	EVP_PKEY *pkey;
+	int ret;
 
 	if (rsa == NULL)
 		{
@@ -272,31 +192,91 @@ RSA *rsa;
 
         if ((ssl->cert == NULL) || (ssl->cert == ssl->ctx->default_cert))
                 {
-                if (ssl->cert != NULL) ssl_cert_free(ssl->cert);
                 c=ssl_cert_new();
 		if (c == NULL)
 			{
 			SSLerr(SSL_F_SSL_USE_RSAPRIVATEKEY,ERR_R_MALLOC_FAILURE);
 			return(0);
 			}
+                if (ssl->cert != NULL) ssl_cert_free(ssl->cert);
 		ssl->cert=c;
 		}
 	c=ssl->cert;
-
-	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE)) return(0);
-	if (c->privatekey) EVP_PKEY_free(c->privatekey);
-	if ((c->privatekey=EVP_PKEY_new()) == NULL)
+	if ((pkey=EVP_PKEY_new()) == NULL)
 		{
 		SSLerr(SSL_F_SSL_USE_RSAPRIVATEKEY,ERR_R_EVP_LIB);
 		return(0);
 		}
-	c->privatekey->type=EVP_PKEY_RSA;
-	c->privatekey->pkey.rsa=rsa;
 
 	CRYPTO_add(&rsa->references,1,CRYPTO_LOCK_RSA);
+	EVP_PKEY_assign_RSA(pkey,rsa);
+
+	ret=ssl_set_pkey(c,pkey);
+	EVP_PKEY_free(pkey);
+	return(ret);
+	}
+#endif
+
+static int ssl_set_pkey(c,pkey)
+CERT *c;
+EVP_PKEY *pkey;
+	{
+	int i,ok=0,bad=0;
+
+	i=ssl_cert_type(NULL,pkey);
+	if (i < 0)
+		{
+		SSLerr(SSL_F_SSL_SET_PKEY,SSL_R_UNKNOWN_CERTIFICATE_TYPE);
+		return(0);
+		}
+
+	if (c->pkeys[i].x509 != NULL)
+		{
+		if (!X509_check_private_key(c->pkeys[i].x509,pkey))
+			{
+			if ((i == SSL_PKEY_DH_RSA) || (i == SSL_PKEY_DH_DSA))
+				{
+				i=(i == SSL_PKEY_DH_RSA)?
+					SSL_PKEY_DH_DSA:SSL_PKEY_DH_RSA;
+
+				if (c->pkeys[i].x509 == NULL)
+					ok=1;
+				else
+					{
+					if (!X509_check_private_key(
+						c->pkeys[i].x509,pkey))
+						bad=1;
+					else
+						ok=1;
+					}
+				}
+			else
+				bad=1;
+			}
+		else
+			ok=1;
+		}
+	else
+		ok=1;
+
+	if (bad)
+		{
+		X509_free(c->pkeys[i].x509);
+		c->pkeys[i].x509=NULL;
+		return(0);
+		}
+
+	if (c->pkeys[i].privatekey != NULL)
+		EVP_PKEY_free(c->pkeys[i].privatekey);
+	CRYPTO_add(&pkey->references,1,CRYPTO_LOCK_EVP_PKEY);
+	c->pkeys[i].privatekey=pkey;
+	c->key= &(c->pkeys[i]);
+
+	c->valid=0;
 	return(1);
 	}
 
+#ifndef NO_RSA
 int SSL_use_RSAPrivateKey_file(ssl, file, type)
 SSL *ssl;
 char *file;
@@ -319,7 +299,7 @@ int type;
 
 	if (BIO_read_filename(in,file) <= 0)
 		{
-		SYSerr(ERR_F_FOPEN,errno);
+		SYSerr(SYS_F_FOPEN,errno);
 		SSLerr(SSL_F_SSL_USE_RSAPRIVATEKEY_FILE,ERR_R_SYS_LIB);
 		goto end;
 		}
@@ -361,7 +341,7 @@ long len;
 	RSA *rsa;
 
 	p=d;
-	if ((rsa=d2i_RSAPrivateKey(&rsa,&p,(long)len)) == NULL)
+	if ((rsa=d2i_RSAPrivateKey(NULL,&p,(long)len)) == NULL)
 		{
 		SSLerr(SSL_F_SSL_USE_RSAPRIVATEKEY_ASN1,ERR_R_ASN1_LIB);
 		return(0);
@@ -371,12 +351,14 @@ long len;
 	RSA_free(rsa);
 	return(ret);
 	}
+#endif /* !NO_RSA */
 
 int SSL_use_PrivateKey(ssl, pkey)
 SSL *ssl;
 EVP_PKEY *pkey;
 	{
 	CERT *c;
+	int ret;
 
 	if (pkey == NULL)
 		{
@@ -386,22 +368,19 @@ EVP_PKEY *pkey;
 
         if ((ssl->cert == NULL) || (ssl->cert == ssl->ctx->default_cert))
                 {
-                if (ssl->cert != NULL) ssl_cert_free(ssl->cert);
                 c=ssl_cert_new();
 		if (c == NULL)
 			{
 			SSLerr(SSL_F_SSL_USE_PRIVATEKEY,ERR_R_MALLOC_FAILURE);
 			return(0);
 			}
+                if (ssl->cert != NULL) ssl_cert_free(ssl->cert);
 		ssl->cert=c;
 		}
 	c=ssl->cert;
 
-	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE)) return(0);
-	if (c->privatekey) EVP_PKEY_free(c->privatekey);
-	c->privatekey=pkey;
-	CRYPTO_add(&pkey->references,1,CRYPTO_LOCK_EVP_PKEY);
-	return(1);
+	ret=ssl_set_pkey(c,pkey);
+	return(ret);
 	}
 
 int SSL_use_PrivateKey_file(ssl, file, type)
@@ -426,7 +405,7 @@ int type;
 
 	if (BIO_read_filename(in,file) <= 0)
 		{
-		SYSerr(ERR_F_FOPEN,errno);
+		SYSerr(SYS_F_FOPEN,errno);
 		SSLerr(SSL_F_SSL_USE_PRIVATEKEY_FILE,ERR_R_SYS_LIB);
 		goto end;
 		}
@@ -464,7 +443,7 @@ long len;
 	EVP_PKEY *pkey;
 
 	p=d;
-	if ((pkey=d2i_PrivateKey(type,&pkey,&p,(long)len)) == NULL)
+	if ((pkey=d2i_PrivateKey(type,NULL,&p,(long)len)) == NULL)
 		{
 		SSLerr(SSL_F_SSL_USE_PRIVATEKEY_ASN1,ERR_R_ASN1_LIB);
 		return(0);
@@ -480,7 +459,6 @@ SSL_CTX *ctx;
 X509 *x;
 	{
 	CERT *c;
-	EVP_PKEY *pkey;
 
 	if (x == NULL)
 		{
@@ -491,23 +469,81 @@ X509 *x;
 	if (ctx->default_cert == NULL)
 		{
 		c=ssl_cert_new();
-		if (c == NULL) return(0);
+		if (c == NULL)
+			{
+			SSLerr(SSL_F_SSL_CTX_USE_CERTIFICATE,ERR_R_MALLOC_FAILURE);
+			return(0);
+			}
 		ctx->default_cert=c;
 		}
 	c=ctx->default_cert;
 
-	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE)) return(0);
-	CRYPTO_add(&x->references,1,CRYPTO_LOCK_X509);
-	if (c->x509 != NULL) X509_free(c->x509);
-	c->x509=x;
-	if (c->publickey != NULL) EVP_PKEY_free(c->publickey);
+	return(ssl_set_cert(c,x));
+	}
+
+static int ssl_set_cert(c,x)
+CERT *c;
+X509 *x;
+	{
+	EVP_PKEY *pkey;
+	int i,ok=0,bad=0;
+
 	pkey=X509_get_pubkey(x);
 	if (pkey == NULL)
 		{
-		SSLerr(SSL_F_SSL_CTX_USE_CERTIFICATE,SSL_R_X509_LIB);
+		SSLerr(SSL_F_SSL_SET_PKEY,SSL_R_X509_LIB);
 		return(0);
 		}
-	c->publickey=pkey;
+
+	i=ssl_cert_type(x,pkey);
+	if (i < 0)
+		{
+		SSLerr(SSL_F_SSL_SET_PKEY,SSL_R_UNKNOWN_CERTIFICATE_TYPE);
+		return(0);
+		}
+
+	if (c->pkeys[i].privatekey != NULL)
+		{
+		if (!X509_check_private_key(x,c->pkeys[i].privatekey))
+			{
+			if ((i == SSL_PKEY_DH_RSA) || (i == SSL_PKEY_DH_DSA))
+				{
+				i=(i == SSL_PKEY_DH_RSA)?
+					SSL_PKEY_DH_DSA:SSL_PKEY_DH_RSA;
+
+				if (c->pkeys[i].privatekey == NULL)
+					ok=1;
+				else
+					{
+					if (!X509_check_private_key(x,
+						c->pkeys[i].privatekey))
+						bad=1;
+					else
+						ok=1;
+					}
+				}
+			else
+				bad=1;
+			}
+		else
+			ok=1;
+		}
+	else
+		ok=1;
+
+	if (bad)
+		{
+		EVP_PKEY_free(c->pkeys[i].privatekey);
+		c->pkeys[i].privatekey=NULL;
+		}
+
+	if (c->pkeys[i].x509 != NULL)
+		X509_free(c->pkeys[i].x509);
+	CRYPTO_add(&x->references,1,CRYPTO_LOCK_X509);
+	c->pkeys[i].x509=x;
+	c->key= &(c->pkeys[i]);
+
+	c->valid=0;
 	return(1);
 	}
 
@@ -534,7 +570,7 @@ int type;
 
 	if (BIO_read_filename(in,file) <= 0)
 		{
-		SYSerr(ERR_F_FOPEN,errno);
+		SYSerr(SYS_F_FOPEN,errno);
 		SSLerr(SSL_F_SSL_CTX_USE_CERTIFICATE_FILE,ERR_R_SYS_LIB);
 		goto end;
 		}
@@ -561,7 +597,6 @@ int type;
 		}
 
 	ret=SSL_CTX_use_certificate(ctx,x);
-	ret=1;
 end:
 	if (x != NULL) X509_free(x);
 	if (in != NULL) BIO_free(in);
@@ -588,11 +623,14 @@ unsigned char *d;
 	return(ret);
 	}
 
+#ifndef NO_RSA
 int SSL_CTX_use_RSAPrivateKey(ctx, rsa)
 SSL_CTX *ctx;
 RSA *rsa;
 	{
+	int ret;
 	CERT *c;
+	EVP_PKEY *pkey;
 
 	if (rsa == NULL)
 		{
@@ -611,18 +649,18 @@ RSA *rsa;
 		}
 	c=ctx->default_cert;
 
-	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE)) return(0);
-	if (c->privatekey) EVP_PKEY_free(c->privatekey);
-	if ((c->privatekey=EVP_PKEY_new()) == NULL)
+	if ((pkey=EVP_PKEY_new()) == NULL)
 		{
 		SSLerr(SSL_F_SSL_CTX_USE_RSAPRIVATEKEY,ERR_R_EVP_LIB);
 		return(0);
 		}
-	c->privatekey->type=EVP_PKEY_RSA;
-	c->privatekey->pkey.rsa=rsa;
-	CRYPTO_add(&rsa->references,1,CRYPTO_LOCK_RSA);
 
-	return(1);
+	CRYPTO_add(&rsa->references,1,CRYPTO_LOCK_RSA);
+	EVP_PKEY_assign_RSA(pkey,rsa);
+
+	ret=ssl_set_pkey(c,pkey);
+	EVP_PKEY_free(pkey);
+	return(ret);
 	}
 
 int SSL_CTX_use_RSAPrivateKey_file(ctx, file, type)
@@ -647,7 +685,7 @@ int type;
 
 	if (BIO_read_filename(in,file) <= 0)
 		{
-		SYSerr(ERR_F_FOPEN,errno);
+		SYSerr(SYS_F_FOPEN,errno);
 		SSLerr(SSL_F_SSL_CTX_USE_RSAPRIVATEKEY_FILE,ERR_R_SYS_LIB);
 		goto end;
 		}
@@ -689,7 +727,7 @@ long len;
 	RSA *rsa;
 
 	p=d;
-	if ((rsa=d2i_RSAPrivateKey(&rsa,&p,(long)len)) == NULL)
+	if ((rsa=d2i_RSAPrivateKey(NULL,&p,(long)len)) == NULL)
 		{
 		SSLerr(SSL_F_SSL_CTX_USE_RSAPRIVATEKEY_ASN1,ERR_R_ASN1_LIB);
 		return(0);
@@ -699,6 +737,7 @@ long len;
 	RSA_free(rsa);
 	return(ret);
 	}
+#endif /* !NO_RSA */
 
 int SSL_CTX_use_PrivateKey(ctx, pkey)
 SSL_CTX *ctx;
@@ -724,11 +763,7 @@ EVP_PKEY *pkey;
 		}
 	c=ctx->default_cert;
 
-	if (!ssl_set_cert_type(c,SSL_CT_X509_CERTIFICATE)) return(0);
-	if (c->privatekey) EVP_PKEY_free(c->privatekey);
-	c->privatekey=pkey;
-	CRYPTO_add(&pkey->references,1,CRYPTO_LOCK_EVP_PKEY);
-	return(1);
+	return(ssl_set_pkey(c,pkey));
 	}
 
 int SSL_CTX_use_PrivateKey_file(ctx, file, type)
@@ -753,7 +788,7 @@ int type;
 
 	if (BIO_read_filename(in,file) <= 0)
 		{
-		SYSerr(ERR_F_FOPEN,errno);
+		SYSerr(SYS_F_FOPEN,errno);
 		SSLerr(SSL_F_SSL_CTX_USE_PRIVATEKEY_FILE,ERR_R_SYS_LIB);
 		goto end;
 		}
@@ -791,7 +826,7 @@ long len;
 	EVP_PKEY *pkey;
 
 	p=d;
-	if ((pkey=d2i_PrivateKey(type,&pkey,&p,(long)len)) == NULL)
+	if ((pkey=d2i_PrivateKey(type,NULL,&p,(long)len)) == NULL)
 		{
 		SSLerr(SSL_F_SSL_CTX_USE_PRIVATEKEY_ASN1,ERR_R_ASN1_LIB);
 		return(0);
