@@ -11,7 +11,7 @@ $#ARGV == 0 || print STDERR "usage: perlpath newpath  (eg /usr/bin)\n";
 
 sub wanted
 	{
-	return unless /\.pl$/ || /^Configure$/;
+	return unless /\.pl$/ || /^[Cc]onfigur/;
 
 	open(IN,"<$_") || die "unable to open $dir/$_:$!\n";
 	@a=<IN>;
@@ -19,9 +19,12 @@ sub wanted
 
 	$a[0]="#!$ARGV[0]/perl\n";
 
-	# I'm not going to play safe, but then people should not be
-	# running this often.
-	open(OUT,">$_") || die "unable to open $dir/$_:$!\n";
+	# Playing it safe...
+	$new="$_.new";
+	open(OUT,">$new") || die "unable to open $dir/$new:$!\n";
 	print OUT @a;
 	close(OUT);
+
+	rename($new,$_) || die "unable to rename $dir/$new:$!\n";
+	chmod(0755,$_) || die "unable to chmod $dir/$new:$!\n";
 	}
